@@ -44,6 +44,7 @@
 #include <fcntl.h>
 #include <utime.h>
 #include <limits.h>
+#include <errno.h>
 #include <sys/time.h>
 
 #ifdef HAVE_POPT
@@ -159,7 +160,7 @@ static int
 get_path_for_file (const char *folder, CameraFile *file, char **path)
 {
 	unsigned int i, l;
-	int n;
+	static int filenr = 0;
 	char *s, b[1024];
 	const char *name, *prefix;
 	CameraFileType type;
@@ -250,35 +251,21 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 			}
 			switch (gp_params.filename[i]) {
 			case 'n':
-
 				/*
-				 * Get the number of the file. This can only
-				 * be done with persistent files!
+ 				 * Previously this used an folder index number.
+				 * Now this uses a linear increasing number.
 				 */
-				if (!folder) {
-					gp_context_error (gp_params.context, 
-						_("You cannot use '%%n' "
-						  "in combination with "
-						  "non-persistent files!"));
-					return (GP_ERROR_BAD_PARAMETERS);
-				}
-				n = gp_filesystem_number (gp_params.camera->fs,
-					folder, name, gp_params.context);
-				if (n < 0) {
-					free (*path);
-					*path = NULL;
-					return (n);
-				}
 				if (precision > 1) {
 					char padfmt[16];
 					strcpy(padfmt, "%!.*i");
 					padfmt[1] = padding;
 					snprintf (b, sizeof (b), padfmt,
-						  precision, n + 1);
+						  precision, filenr);
 				} else {
 					snprintf (b, sizeof (b), "%i",
-						  n + 1);
+						  filenr);
 				}
+				filenr++;
 				break;
 
 			case 'C':
@@ -516,6 +503,10 @@ save_file_to_file (Camera *camera, GPContext *context, Flags flags,
 	strcpy (tmpname, "tmpfileXXXXXX");
 	fd = mkstemp(tmpname);
 	if (fd == -1) {
+		if (errno == EACCES) {
+			gp_context_error (context, _("Permission denied"));
+			return GP_ERROR;
+		}
         	CR (gp_file_new (&file));
 		tmpfilename = NULL;
 	} else {
@@ -687,8 +678,8 @@ wait_and_handle_event (long waittime, CameraEventType *type, int download) {
 	CameraFilePath	*path;
 
 	if (!type) type = &evtype;
-	data = NULL;
 	evtype = GP_EVENT_UNKNOWN;
+	data = NULL;
 	result = gp_camera_wait_for_event(gp_params.camera, waittime, type, &data, gp_params.context);
 	if (result == GP_ERROR_NOT_SUPPORTED) {
 		*type = GP_EVENT_TIMEOUT;
@@ -787,7 +778,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 			gettimeofday (&expose_end_time, NULL);
 			expose_end_time.tv_sec += glob_bulblength;
 			waittime = timediff_now (&expose_end_time);
-			while (waittime > 0) {
+			while(waittime > 0) {
 				result = wait_and_handle_event(waittime, &evtype, download);
 				if (result != GP_OK)
 					return result;
@@ -1096,6 +1087,7 @@ typedef enum {
 	ARG_HELP,
 	ARG_HOOK_SCRIPT,
 	ARG_LIST_CAMERAS,
+	ARG_LIST_ALL_CONFIG,
 	ARG_LIST_CONFIG,
 	ARG_LIST_FILES,
 	ARG_LIST_FOLDERS,
@@ -1460,6 +1452,9 @@ cb_arg_run (poptContext __unused__ ctx,
 		gp_params.multi_type = MULTI_UPLOAD_META;
 		params->p.r = action_camera_upload_metadata (&gp_params, gp_params.folder, arg);
 		break;
+	case ARG_LIST_ALL_CONFIG:
+		params->p.r = list_all_config_action (&gp_params);
+		break;
 	case ARG_LIST_CONFIG:
 		params->p.r = list_config_action (&gp_params);
 		break;
@@ -1609,8 +1604,7 @@ report_failure (int result, int argc, char **argv)
 			  "gphoto2 as follows:\n\n"));
 
 		/*
-		 * Print the exact command line to assist
-		 * l^Husers
+		 * Print the exact command line to assist bugreporters
 		 */
 		printf ("    env LANG=C gphoto2 --debug --debug-logfile=my-logfile.txt");
 		for (n = 1; n < argc; n++) {
@@ -1620,8 +1614,8 @@ report_failure (int result, int argc, char **argv)
 				printf(" \"%s\"",argv[n]);
 		}
 		printf ("\n\n");
-		printf ("Please make sure there is sufficient quoting "
-			"around the arguments.\n\n");
+		printf (_("Please make sure there is sufficient quoting "
+			"around the arguments.\n\n"));
 	}
 }
 
@@ -1733,6 +1727,8 @@ main (int argc, char **argv, char **envp)
 #endif
 		{"list-config", '\0', POPT_ARG_NONE, NULL, ARG_LIST_CONFIG,
 		 N_("List configuration tree"), NULL},
+		{"list-all-config", '\0', POPT_ARG_NONE, NULL, ARG_LIST_ALL_CONFIG,
+		 N_("Dump full configuration tree"), NULL},
 		{"get-config", '\0', POPT_ARG_STRING, NULL, ARG_GET_CONFIG,
 		 N_("Get configuration value"), NULL},
 		{"set-config", '\0', POPT_ARG_STRING, NULL, ARG_SET_CONFIG,
@@ -1814,7 +1810,7 @@ main (int argc, char **argv, char **envp)
 		{"delete-file", 'd', POPT_ARG_STRING, NULL, ARG_DELETE_FILE,
 		 N_("Delete files given in range"), N_("RANGE")},
 		{"delete-all-files", 'D', POPT_ARG_NONE, NULL,
-		 ARG_DELETE_ALL_FILES, N_("Delete all files in folder"), NULL},
+		 ARG_DELETE_ALL_FILES, N_("Delete all files in folder (--no-recurse by default)"), NULL},
 		{"upload-file", 'u', POPT_ARG_STRING, NULL, ARG_UPLOAD_FILE,
 		 N_("Upload a file to camera"), N_("FILENAME")},
 		{"filename", '\0', POPT_ARG_STRING, NULL, ARG_FILENAME,
